@@ -26,20 +26,24 @@
 #include <grp.h>
 #include <regex.h>
 #include <errno.h>
+#include <getopt.h>
 
+// EXIT_SUCCESS is 0
+// EXIT_FAILURE is 1
+#define EXIT_USAGE 2
+#define EXIT_FATAL 3
+#define XALLOC_EXIT_CODE EXIT_FATAL
+
+#include "c.h"
+#include "nls.h"
+#include "xalloc.h"
 #include "proc/readproc.h"
 #include "proc/sig.h"
 #include "proc/devname.h"
 #include "proc/sysinfo.h"
 #include "proc/version.h" /* procps_version */
 
-// EXIT_SUCCESS is 0
-// EXIT_FAILURE is 1
-#define EXIT_USAGE 2
-#define EXIT_FATAL 3
-
 static int i_am_pkill = 0;
-static const char *progname = "pgrep";
 
 union el {
 	long	num;
@@ -54,6 +58,7 @@ static int opt_oldest = 0;
 static int opt_newest = 0;
 static int opt_negate = 0;
 static int opt_exact = 0;
+static int opt_count = 0;
 static int opt_signal = SIGTERM;
 static int opt_lock = 0;
 static int opt_case = 0;
@@ -70,27 +75,47 @@ static union el *opt_ruid = NULL;
 static char *opt_pattern = NULL;
 static char *opt_pidfile = NULL;
 
-static int usage (int opt) NORETURN;
-static int usage (int opt)
+static int __attribute__ ((__noreturn__)) usage(int opt)
 {
-	int err = (opt=='?'); /* getopt() uses '?' to mark an error */
+	int err = (opt == '?');
 	FILE *fp = err ? stderr : stdout;
 
-	if (i_am_pkill)
-		fprintf (fp, "Usage: pkill [-SIGNAL] [-fvx] ");
-	else
-		fprintf (fp, "Usage: pgrep [-flvx] [-d DELIM] ");
-	fprintf (fp, "[-n|-o] [-P PPIDLIST] [-g PGRPLIST] [-s SIDLIST]\n"
-		 "\t[-u EUIDLIST] [-U UIDLIST] [-G GIDLIST] [-t TERMLIST] "
-		 "[PATTERN]\n");
+	fputs(USAGE_HEADER, fp);
+	fprintf(fp, _(" %s [options] <pattern>\n"), program_invocation_short_name);
+	fputs(USAGE_OPTIONS, fp);
+	if (i_am_pkill == 0) {
+		fputs(_(" -c, --count               count of matching processes\n"
+			" -d, --delimeter <string>  specify output delimeter\n"
+			" -l, --list-name           list PID and process name\n"), fp);
+	}
+	if (i_am_pkill == 1) {
+		fputs(_(" -<sig>, --signal <sig>    signal to send (either number or name)\n"), fp);
+	}
+	fputs(_(" -f, --full                use full process name to match\n"
+		" -g, --pgroup <id,...>     match listed process group IDs\n"
+		" -G, --group <gid,...>     match real group IDs\n"
+		" -n, --newest              select most recently started\n"
+		" -o, --oldest              select least recently started\n"
+		" -P, --parent <ppid,...>   match only childs of given parent\n"
+		" -s, --session <sid,...>   match session IDs\n"
+		" -t, --terminal <tty,...>  match by controlling terminal\n"
+		" -u, --euid <id,...>       match by effective IDs\n"
+		" -U, --uid <id,...>        match by real IDs\n"
+		" -v, --inverse             negates the matching\n"
+		" -x, --exact               match exectly with command name\n"
+		" -F, --pidfile <file>      read PIDs from file\n"
+		" -L, --logpidfile          fail if PID file is not locked\n"), fp);
+	fputs(USAGE_SEPARATOR, fp);
+	fputs(USAGE_HELP, fp);
+	fputs(USAGE_VERSION, fp);
+	fprintf(fp, USAGE_MAN_TAIL("pgrep(1)"));
 
-	exit(err ? EXIT_USAGE : EXIT_SUCCESS);
+	exit(fp == stderr ? EXIT_FAILURE : EXIT_SUCCESS);
 }
 
-
 static union el *split_list (const char *restrict str, int (*convert)(const char *, union el *))
 {
-	char *copy = strdup (str);
+	char *copy = xstrdup (str);
 	char *ptr = copy;
 	char *sep_pos;
 	int i = 0;
@@ -101,9 +126,7 @@ static union el *split_list (const char *restrict str, int (*convert)(const char
 		if (i == size) {
 			size = size * 5 / 4 + 4;
 			// add 1 because slot zero is a count
-			list = realloc (list, 1 + size * sizeof *list);
-			if (list == NULL)
-				exit (EXIT_FATAL);
+			list = xrealloc (list, 1 + size * sizeof *list);
 		}
 		sep_pos = strchr (ptr, ',');
 		if (sep_pos)
@@ -214,7 +237,7 @@ static union el *read_pidfile(void)
 		goto out;
 	if(*endp && !isspace(*endp))
 		goto out;
-	list = malloc(2 * sizeof *list);
+	list = xmalloc(2 * sizeof *list);
 	list[0].num = 1;
 	list[1].num = pid;
 out:
@@ -232,8 +255,7 @@ static int conv_uid (const char *restrict name, union el *restrict e)
 
 	pwd = getpwnam (name);
 	if (pwd == NULL) {
-		fprintf (stderr, "%s: invalid user name: %s\n",
-			 progname, name);
+		xwarnx(_("invalid user name: %s"), name);
 		return 0;
 	}
 	e->num = pwd->pw_uid;
@@ -250,8 +272,7 @@ static int conv_gid (const char *restrict name, union el *restrict e)
 
 	grp = getgrnam (name);
 	if (grp == NULL) {
-		fprintf (stderr, "%s: invalid group name: %s\n",
-			 progname, name);
+		xwarnx(_("invalid group name: %s"), name);
 		return 0;
 	}
 	e->num = grp->gr_gid;
@@ -262,8 +283,7 @@ static int conv_gid (const char *restrict name, union el *restrict e)
 static int conv_pgrp (const char *restrict name, union el *restrict e)
 {
 	if (! strict_atol (name, &e->num)) {
-		fprintf (stderr, "%s: invalid process group: %s\n",
-			 progname, name);
+		xwarnx(_("invalid process group: %s"), name);
 		return 0;
 	}
 	if (e->num == 0)
@@ -275,8 +295,7 @@ static int conv_pgrp (const char *restrict name, union el *restrict e)
 static int conv_sid (const char *restrict name, union el *restrict e)
 {
 	if (! strict_atol (name, &e->num)) {
-		fprintf (stderr, "%s: invalid session id: %s\n",
-			 progname, name);
+		xwarnx(_("invalid session id: %s"), name);
 		return 0;
 	}
 	if (e->num == 0)
@@ -288,8 +307,7 @@ static int conv_sid (const char *restrict name, union el *restrict e)
 static int conv_num (const char *restrict name, union el *restrict e)
 {
 	if (! strict_atol (name, &e->num)) {
-		fprintf (stderr, "%s: not a number: %s\n",
-			 progname, name);
+		xwarnx(_("not a number: %s"), name);
 		return 0;
 	}
 	return 1;
@@ -298,7 +316,7 @@ static int conv_num (const char *restrict name, union el *restrict e)
 
 static int conv_str (const char *restrict name, union el *restrict e)
 {
-	e->str = strdup (name);
+	e->str = xstrdup (name);
 	return 1;
 }
 
@@ -372,9 +390,7 @@ static PROCTAB *do_openproc (void)
 	if (opt_euid && !opt_negate) {
 		int num = opt_euid[0].num;
 		int i = num;
-		uid_t *uids = malloc (num * sizeof (uid_t));
-		if (uids == NULL)
-			exit (EXIT_FATAL);
+		uid_t *uids = xmalloc (num * sizeof (uid_t));
 		while (i-- > 0) {
 			uids[i] = opt_euid[i+1].num;
 		}
@@ -395,13 +411,9 @@ static regex_t * do_regcomp (void)
 		char errbuf[256];
 		int re_err;
 
-		preg = malloc (sizeof (regex_t));
-		if (preg == NULL)
-			exit (EXIT_FATAL);
+		preg = xmalloc (sizeof (regex_t));
 		if (opt_exact) {
-			re = malloc (strlen (opt_pattern) + 5);
-			if (re == NULL)
-				exit (EXIT_FATAL);
+			re = xmalloc (strlen (opt_pattern) + 5);
 			sprintf (re, "^(%s)$", opt_pattern);
 		} else {
 		 	re = opt_pattern;
@@ -434,7 +446,8 @@ static union el * select_procs (int *num)
 	preg = do_regcomp();
 
 	if (opt_newest) saved_start_time =  0ULL;
-	if (opt_oldest) saved_start_time = ~0ULL;
+	else saved_start_time = ~0ULL;
+
 	if (opt_newest) saved_pid = 0;
 	if (opt_oldest) saved_pid = INT_MAX;
 	
@@ -518,14 +531,12 @@ static union el * select_procs (int *num)
 			}
 			if (matches == size) {
 				size = size * 5 / 4 + 4;
-				list = realloc(list, size * sizeof *list);
-				if (list == NULL)
-					exit (EXIT_FATAL);
+				list = xrealloc(list, size * sizeof *list);
 			}
 			if (opt_long) {
 				char buff[5096];  // FIXME
 				sprintf (buff, "%d %s", task.XXXID, cmd);
-				list[matches++].str = strdup (buff);
+				list[matches++].str = xstrdup (buff);
 			} else {
 				list[matches++].num = task.XXXID;
 			}
@@ -545,9 +556,35 @@ static void parse_opts (int argc, char **argv)
 	int opt;
 	int criteria_count = 0;
 
-	if (strstr (argv[0], "pkill")) {
-		i_am_pkill = 1;
-		progname = "pkill";
+	enum {
+		SIGNAL_OPTION = CHAR_MAX + 1
+	};
+	static const struct option longopts[] = {
+		{"signal", required_argument, NULL, SIGNAL_OPTION},
+		{"count", no_argument, NULL, 'c'},
+		{"delimeter", required_argument, NULL, 'd'},
+		{"list-name", no_argument, NULL, 'l'},
+		{"full", no_argument, NULL, 'f'},
+		{"pgroup", required_argument, NULL, 'g'},
+		{"group", required_argument, NULL, 'G'},
+		{"newest", no_argument, NULL, 'n'},
+		{"oldest", no_argument, NULL, 'o'},
+		{"parent", required_argument, NULL, 'P'},
+		{"session", required_argument, NULL, 's'},
+		{"terminal", required_argument, NULL, 't'},
+		{"euid", required_argument, NULL, 'u'},
+		{"uid", required_argument, NULL, 'U'},
+		{"inverse", no_argument, NULL, 'v'},
+		{"exact", no_argument, NULL, 'x'},
+		{"pidfile", required_argument, NULL, 'F'},
+		{"logpidfile", no_argument, NULL, 'L'},
+		{"help", no_argument, NULL, 'h'},
+		{"version", no_argument, NULL, 'V'},
+		{NULL, 0, NULL, 0}
+	};
+
+	if (strstr (program_invocation_short_name, "pkill")) {
+	        i_am_pkill = 1;
 		/* Look for a signal name or number as first argument */
 		if (argc > 1 && argv[1][0] == '-') {
 			int sig;
@@ -564,17 +601,22 @@ static void parse_opts (int argc, char **argv)
 		}
 	} else {
 		/* These options are for pgrep only */
-		strcat (opts, "ld:");
+		strcat (opts, "cld:");
 	}
 			
-	strcat (opts, "LF:fnovxP:g:s:u:U:G:t:?V");
+	strcat (opts, "LF:fnovxP:g:s:u:U:G:t:?Vh");
 	
-	while ((opt = getopt (argc, argv, opts)) != -1) {
+	while ((opt = getopt_long (argc, argv, opts, longopts, NULL)) != -1) {
 		switch (opt) {
+		case SIGNAL_OPTION:
+			opt_signal = signal_name_to_number (optarg);
+			if (opt_signal == -1 && isdigit (optarg[0]))
+				opt_signal = atoi (optarg);
+			break;
 //		case 'D':   // FreeBSD: print info about non-matches for debugging
 //			break;
 		case 'F':   // FreeBSD: the arg is a file containing a PID to match
-			opt_pidfile = strdup (optarg);
+			opt_pidfile = xstrdup (optarg);
 			++criteria_count;
 			break;
 		case 'G':   // Solaris: match rgid/rgroup
@@ -611,12 +653,15 @@ static void parse_opts (int argc, char **argv)
 			++criteria_count;
 			break;
 		case 'V':
-			fprintf(stdout, "%s (%s)\n", progname, procps_version);
+			printf(PROCPS_NG_VERSION);
 			exit(EXIT_SUCCESS);
 //		case 'c':   // Solaris: match by contract ID
 //			break;
+		case 'c':
+			opt_count = 1;
+			break;
 		case 'd':   // Solaris: change the delimiter
-			opt_delim = strdup (optarg);
+			opt_delim = xstrdup (optarg);
 			break;
 		case 'f':   // Solaris: match full process name (as in "ps -f")
 			opt_full = 1;
@@ -678,58 +723,73 @@ static void parse_opts (int argc, char **argv)
 			break;
 //		case 'z':   // Solaris: match by zone ID
 //			break;
-		case '?':
+		case 'h':
 			usage (opt);
 			break;
+		case '?':
+			usage (optopt ? optopt : opt);
+			break;
 		}
 	}
 
-	if(opt_lock && !opt_pidfile){
-		fprintf(stderr, "%s: -L without -F makes no sense\n",progname);
-		usage(0);
-	}
+	if(opt_lock && !opt_pidfile)
+		xerrx(EXIT_FAILURE, _("-L without -F makes no sense\n"
+                                     "Try `%s --help' for more information."),
+                                     program_invocation_short_name);
 
 	if(opt_pidfile){
 		opt_pid = read_pidfile();
-		if(!opt_pid){
-			fprintf(stderr, "%s: pidfile not valid\n",progname);
-			usage(0);
-		}
+		if(!opt_pid)
+			xerrx(EXIT_FAILURE, _("pidfile not valid\n"
+			                     "Try `%s --help' for more information."),
+			                     program_invocation_short_name);
 	}
 
         if (argc - optind == 1)
 		opt_pattern = argv[optind];
 	else if (argc - optind > 1)
-		usage (0);
-	else if (criteria_count == 0) {
-		fprintf (stderr, "%s: No matching criteria specified\n",
-			 progname);
-		usage (0);
-	}
+		xerrx(EXIT_FAILURE, _("only one pattern can be provided\n"
+		                     "Try `%s --help' for more information."),
+		                     program_invocation_short_name);
+	else if (criteria_count == 0)
+		xerrx(EXIT_FAILURE, _("no matching criteria specified\n"
+                                     "Try `%s --help' for more information."),
+                                     program_invocation_short_name);
 }
 
 
-int main (int argc, char *argv[])
+int main (int argc, char **argv)
 {
 	union el *procs;
 	int num;
 
+	program_invocation_name = program_invocation_short_name;
+	setlocale (LC_ALL, "");
+	bindtextdomain(PACKAGE, LOCALEDIR);
+	textdomain(PACKAGE);
+
 	parse_opts (argc, argv);
 
 	procs = select_procs (&num);
 	if (i_am_pkill) {
 		int i;
 		for (i = 0; i < num; i++) {
-			if (kill (procs[i].num, opt_signal) != -1) continue;
-			if (errno==ESRCH) continue; // gone now, which is OK
-			fprintf (stderr, "pkill: %ld - %s\n",
-				 procs[i].num, strerror (errno));
+			if (kill (procs[i].num, opt_signal) != -1)
+			        continue;
+			if (errno==ESRCH)
+				 // gone now, which is OK
+			        continue;
+			xwarn(_("killing pid %ld failed"));
 		}
 	} else {
-		if (opt_long)
-			output_strlist(procs,num);
-		else
-			output_numlist(procs,num);
+		if (opt_count) {
+			fprintf(stdout, "%ld\n", num);
+		} else {
+			if (opt_long)
+				output_strlist (procs,num);
+			else
+				output_numlist (procs,num);
+		}
 	}
 	return !num; // exit(EXIT_SUCCESS) if match, otherwise exit(EXIT_FAILURE)
 }
