@@ -42,6 +42,12 @@ extern void __cyg_profile_func_enter(void*,void*);
 #define IS_THREAD(q) ( q->pad_1 == '\xee' )
 #endif
 
+// utility buffers of MAX_BUFSZ bytes each, available to
+// any function following an openproc() call
+static char *src_buffer,
+            *dst_buffer;
+#define MAX_BUFSZ 1024*64*2
+
 #ifndef SIGNAL_STRING
 // convert hex string to unsigned long long
 static unsigned long long unhex(const char *restrict cp){
@@ -212,7 +218,7 @@ ENTER(0x220);
         // examine a field name (hash and compare)
     base:
         if(unlikely(!*S)) break;
-        entry = table[63 & (asso[S[3]] + asso[S[2]] + asso[S[0]])];
+        entry = table[63 & (asso[(int)S[3]] + asso[(int)S[2]] + asso[(int)S[0]])];
         colon = strchr(S, ':');
         if(unlikely(!colon)) break;
         if(unlikely(colon[1]!='\t')) break;
@@ -345,7 +351,7 @@ ENTER(0x220);
                 if (' '  == P->supgid[j])
                     P->supgid[j] = ',';
         } else
-            P->supgid = strdup("-");
+            P->supgid = xstrdup("-");
         continue;
     }
     case_CapBnd:
@@ -402,7 +408,7 @@ static void supgrps_from_supgids (proc_t *p) {
     int t;
 
     if (!p->supgid || '-' == *p->supgid) {
-        p->supgrp = strdup("-");
+        p->supgrp = xstrdup("-");
         return;
     }
     s = p->supgid;
@@ -410,7 +416,7 @@ static void supgrps_from_supgids (proc_t *p) {
     do {
         if (',' == *s) ++s;
         g = group_from_gid((uid_t)strtol(s, &s, 10));
-        p->supgrp = realloc(p->supgrp, P_G_SZ+t+2);
+        p->supgrp = xrealloc(p->supgrp, P_G_SZ+t+2);
         t += snprintf(p->supgrp+t, P_G_SZ+2, "%s%s", t ? "," : "", g);
     } while (*s);
 }
@@ -608,6 +614,7 @@ static int read_unvectored(char *restrict const dst, unsigned sz, const char* wh
         int i=n;
         while(i--)
             if(dst[i]=='\n' || dst[i]=='\0') dst[i]=sep;
+        if(dst[n-1]==' ') dst[n-1]='\0';
     }
     dst[n] = '\0';
     return n;
@@ -620,7 +627,7 @@ static char** vectorize_this_str (const char* src) {
 
     tot = strlen(src) + 1;                       // prep for our vectors
     adj = (pSZ-1) - ((tot + pSZ-1) & (pSZ-1));   // calc alignment bytes
-    cpy = xcalloc(NULL, tot + adj + (2 * pSZ));  // get new larger buffer
+    cpy = xcalloc(tot + adj + (2 * pSZ));        // get new larger buffer
     snprintf(cpy, tot, "%s", src);               // duplicate their string
     vec = (char**)(cpy + tot + adj);             // prep pointer to pointers
     *vec = cpy;                                  // point 1st vector to string
@@ -660,14 +667,13 @@ static void fill_cgroup_cvt (const char* directory, proc_t *restrict p) {
     // and guarantees the caller a valid proc_t.cmdline pointer.
 static void fill_cmdline_cvt (const char* directory, proc_t *restrict p) {
  #define uFLG ( ESC_BRACKETS | ESC_DEFUNCT )
-    char sbuf[2048], dbuf[2048];
-    int whackable_int = sizeof(dbuf);
+    int whackable_int = MAX_BUFSZ;
 
-    if (read_unvectored(sbuf, sizeof(sbuf), directory, "cmdline", ' '))
-        escape_str(dbuf, sbuf, sizeof(dbuf), &whackable_int);
+    if (read_unvectored(src_buffer, MAX_BUFSZ, directory, "cmdline", ' '))
+        escape_str(dst_buffer, src_buffer, MAX_BUFSZ, &whackable_int);
     else
-        escape_command(dbuf, p, sizeof(dbuf), &whackable_int, uFLG);
-    p->cmdline = vectorize_this_str(dbuf);
+        escape_command(dst_buffer, p, MAX_BUFSZ, &whackable_int, uFLG);
+    p->cmdline = vectorize_this_str(dst_buffer);
  #undef uFLG
 }
 
@@ -1012,7 +1018,7 @@ proc_t* readproc(PROCTAB *restrict const PT, proc_t *restrict p) {
 //  }
 
   saved_p = p;
-  if(!p) p = xcalloc(NULL, sizeof *p);
+  if(!p) p = xcalloc(sizeof *p);
   else free_acquired(p, 1);
 
   for(;;){
@@ -1041,7 +1047,7 @@ proc_t* readtask(PROCTAB *restrict const PT, const proc_t *restrict const p, pro
   proc_t *saved_t;
 
   saved_t = t;
-  if(!t) t = xcalloc(NULL, sizeof *t);
+  if(!t) t = xcalloc(sizeof *t);
   else free_acquired(t, 1);
 
   // 1. got to fake a thread for old kernels
@@ -1091,14 +1097,14 @@ out:
 // the next unique process or task available.  If no more are available,
 // return a null pointer (boolean false).  Use the passed buffer instead
 // of allocating space if it is non-NULL.
-extern proc_t* readeither (PROCTAB *restrict const PT, proc_t *restrict x) {
+proc_t* readeither (PROCTAB *restrict const PT, proc_t *restrict x) {
     static proc_t skel_p;    // skeleton proc_t, only uses tid + tgid
     static proc_t *new_p;    // for process/task transitions
     char path[PROCPATHLEN];
     proc_t *saved_x, *ret;
 
     saved_x = x;
-    if (!x) x = xcalloc(NULL, sizeof(*x));
+    if (!x) x = xcalloc(sizeof(*x));
     else free_acquired(x,1);
     if (new_p) goto next_task;
 
@@ -1135,9 +1141,9 @@ PROCTAB* openproc(int flags, ...) {
     static int did_stat;
     PROCTAB* PT = xmalloc(sizeof(PROCTAB));
 
-    if(!did_stat){
-      task_dir_missing = stat("/proc/self/task", &sbuf);
-      did_stat = 1;
+    if (!did_stat){
+        task_dir_missing = stat("/proc/self/task", &sbuf);
+        did_stat = 1;
     }
     PT->taskdir = NULL;
     PT->taskdir_user = -1;
@@ -1146,24 +1152,28 @@ PROCTAB* openproc(int flags, ...) {
 
     PT->reader = simple_readproc;
     if (flags & PROC_PID){
-      PT->procfs = NULL;
-      PT->finder = listed_nextpid;
+        PT->procfs = NULL;
+        PT->finder = listed_nextpid;
     }else{
-      PT->procfs = opendir("/proc");
-      if(!PT->procfs) { free(PT); return NULL; }
-      PT->finder = simple_nextpid;
+        PT->procfs = opendir("/proc");
+        if (!PT->procfs) { free(PT); return NULL; }
+        PT->finder = simple_nextpid;
     }
     PT->flags = flags;
 
-    va_start(ap, flags);		/*  Init args list */
+    va_start(ap, flags);
     if (flags & PROC_PID)
-    	PT->pids = va_arg(ap, pid_t*);
-    else if (flags & PROC_UID) {
-    	PT->uids = va_arg(ap, uid_t*);
-	PT->nuid = va_arg(ap, int);
+        PT->pids = va_arg(ap, pid_t*);
+    else if (flags & PROC_UID){
+        PT->uids = va_arg(ap, uid_t*);
+        PT->nuid = va_arg(ap, int);
     }
-    va_end(ap);				/*  Clean up args list */
+    va_end(ap);
 
+    if (!src_buffer){
+        src_buffer = xmalloc(MAX_BUFSZ);
+        dst_buffer = xmalloc(MAX_BUFSZ);
+    }
     return PT;
 }
 
@@ -1191,7 +1201,7 @@ void look_up_our_self(proc_t *p) {
     char sbuf[1024];
 
     if(file2str("/proc/self", "stat", sbuf, sizeof sbuf) == -1){
-        fprintf(stderr, "Error, do this: mount -t proc none /proc\n");
+        fprintf(stderr, "Error, do this: mount -t proc proc /proc\n");
         _exit(47);
     }
     stat2proc(sbuf, p);    // parse /proc/self/stat
@@ -1229,6 +1239,8 @@ proc_t** readproctab(int flags, ...) {
     else
 	PT = openproc(flags);
     va_end(ap);
+    if (!PT)
+      return 0;
     do {					/* read table: */
 	tab = xrealloc(tab, (n+1)*sizeof(proc_t*));/* realloc as we go, using */
 	tab[n] = readproc_direct(PT, NULL);     /* final null to terminate */
@@ -1257,15 +1269,13 @@ proc_data_t *readproctab2(int(*want_proc)(proc_t *buf), int(*want_task)(proc_t *
         if(n_alloc == n_used){
           //proc_t *old = data;
           n_alloc = n_alloc*5/4+30;  // grow by over 25%
-          data = realloc(data,sizeof(proc_t)*n_alloc);
-          //if(!data) return NULL;
+          data = xrealloc(data,sizeof(proc_t)*n_alloc);
           memset(data+n_used, 0, sizeof(proc_t)*(n_alloc-n_used));
         }
         if(n_proc_alloc == n_proc){
           //proc_t **old = ptab;
           n_proc_alloc = n_proc_alloc*5/4+30;  // grow by over 25%
-          ptab = realloc(ptab,sizeof(proc_t*)*n_proc_alloc);
-          //if(!ptab) return NULL;
+          ptab = xrealloc(ptab,sizeof(proc_t*)*n_proc_alloc);
         }
         tmp = readproc_direct(PT, data+n_used);
         if(!tmp) break;
@@ -1277,17 +1287,15 @@ proc_data_t *readproctab2(int(*want_proc)(proc_t *buf), int(*want_task)(proc_t *
           if(n_alloc == n_used){
             proc_t *old = data;
             n_alloc = n_alloc*5/4+30;  // grow by over 25%
-            data = realloc(data,sizeof(proc_t)*n_alloc);
+            data = xrealloc(data,sizeof(proc_t)*n_alloc);
             // have to move tmp too
             tmp = data+(tmp-old);
-            //if(!data) return NULL;
             memset(data+n_used+1, 0, sizeof(proc_t)*(n_alloc-(n_used+1)));
           }
           if(n_task_alloc == n_task){
             //proc_t **old = ttab;
             n_task_alloc = n_task_alloc*5/4+1;  // grow by over 25%
-            ttab = realloc(ttab,sizeof(proc_t*)*n_task_alloc);
-            //if(!ttab) return NULL;
+            ttab = xrealloc(ttab,sizeof(proc_t*)*n_task_alloc);
           }
           t = readtask_direct(PT, tmp, data+n_used);
           if(!t) break;
@@ -1325,7 +1333,7 @@ proc_data_t *readproctab3 (int(*want_task)(proc_t *buf), PROCTAB *restrict const
     for (;;) {
         if (n_alloc == n_used) {
             n_alloc = n_alloc*5/4+30;  // grow by over 25%
-            tab = realloc(tab,sizeof(proc_t*)*n_alloc);
+            tab = xrealloc(tab,sizeof(proc_t*)*n_alloc);
         }
         // let this next guy allocate the necessary proc_t storage
         // (or recycle it) since he can't tolerate realloc relocations
@@ -1348,7 +1356,7 @@ proc_data_t *readproctab3 (int(*want_task)(proc_t *buf), PROCTAB *restrict const
  * and filled out proc_t structure.
  */
 proc_t * get_proc_stats(pid_t pid, proc_t *p) {
-	static char path[PATH_MAX], sbuf[1024];
+	static char path[32], sbuf[1024];
 	struct stat statbuf;
 
 	sprintf(path, "/proc/%d", pid);
@@ -1369,3 +1377,4 @@ proc_t * get_proc_stats(pid_t pid, proc_t *p) {
 
 #undef MK_THREAD
 #undef IS_THREAD
+#undef MAX_BUFSZ
