@@ -13,21 +13,21 @@
  * #define Print(fmt, args...) printf("Debug: " fmt, ## args)
  */
 
-#include <stdlib.h>
-#include <stdio.h>
-#include <string.h>
-
-/* username lookups */
-#include <sys/types.h>
-#include <pwd.h>
 #include <grp.h>
-
-#include <sys/stat.h>
+#include <pwd.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
 #include <unistd.h>
 
-#include "common.h"
+#include <sys/stat.h>
+#include <sys/types.h>
+
+#include "../proc/alloc.h"
 #include "../proc/version.h"
 
+#include "common.h"
+
 #define ARG_GNU  0
 #define ARG_END  1
 #define ARG_PGRP 2
@@ -46,9 +46,8 @@ static char *flagptr;  /* current location in ps_argv[thisarg] */
 static int not_pure_unix = 0;  /* set by BSD and GNU options */
 static int force_bsd = 0;  /* set when normal parsing fails */
 
-#define exclusive(x) if((ps_argc != 2) || strcmp(ps_argv[1],x))\
-  return "The " x " option is exclusive."
-
+#define exclusive(x) if((ps_argc != 2) || strcmp(ps_argv[1],x)) \
+  return _("the option is exclusive: " x)
 
 /********** utility functions **********/
 
@@ -73,12 +72,10 @@ static const char *get_opt_arg(void){
 static const char *parse_pid(char *str, sel_union *ret){
   char *endp;
   unsigned long num;
-  static const char pidrange[]  = "Process ID out of range.";
-  static const char pidsyntax[] = "Process ID list syntax error.";
   num = strtoul(str, &endp, 0);
-  if(*endp != '\0')      return pidsyntax;
-  if(num<1)              return pidrange;
-  if(num > 0x7fffffffUL) return pidrange;
+  if(*endp != '\0')      return _("Process ID list syntax error.");
+  if(num<1)              return _("Process ID out of range.");
+  if(num > 0x7fffffffUL) return _("Process ID out of range.");
   ret->pid = num;
   return 0;
 }
@@ -87,15 +84,13 @@ static const char *parse_uid(char *str, sel_union *ret){
   struct passwd *passwd_data;
   char *endp;
   unsigned long num;
-  static const char uidrange[] = "User ID out of range.";
-  static const char uidexist[] = "User name does not exist.";
   num = strtoul(str, &endp, 0);
   if(*endp != '\0'){  /* hmmm, try as login name */
     passwd_data = getpwnam(str);
-    if(!passwd_data)    return uidexist;
+    if(!passwd_data)    return _("User name does not exist.");
     num = passwd_data->pw_uid;
   }
-  if(num > 0xfffffffeUL) return uidrange;
+  if(num > 0xfffffffeUL) return _("User ID out of range.");
   ret->uid = num;
   return 0;
 }
@@ -104,15 +99,13 @@ static const char *parse_gid(char *str, sel_union *ret){
   struct group *group_data;
   char *endp;
   unsigned long num;
-  static const char gidrange[] = "Group ID out of range.";
-  static const char gidexist[] = "Group name does not exist.";
   num = strtoul(str, &endp, 0);
   if(*endp != '\0'){  /* hmmm, try as login name */
     group_data = getgrnam(str);
-    if(!group_data)    return gidexist;
+    if(!group_data)    return _("Group name does not exist.");
     num = group_data->gr_gid;
   }
-  if(num > 0xfffffffeUL) return gidrange;
+  if(num > 0xfffffffeUL) return _("Group ID out of range.");
   ret->gid = num;
   return 0;
 }
@@ -125,12 +118,10 @@ static const char *parse_cmd(char *str, sel_union *ret){
 
 static const char *parse_tty(char *str, sel_union *ret){
   struct stat sbuf;
-  static const char missing[] = "TTY could not be found.";
-  static const char not_tty[] = "List member was not a TTY.";
   char path[4096];
   if(str[0]=='/'){
     if(stat(str, &sbuf) >= 0) goto found_it;
-    return missing;
+    return _("TTY could not be found.");;
   }
 #define lookup(p) \
   snprintf(path,4096,p,str); \
@@ -154,9 +145,9 @@ static const char *parse_tty(char *str, sel_union *ret){
     return 0;
   }
 #undef lookup
-  return missing;
+  return _("TTY could not be found.");;
 found_it:
-  if(!S_ISCHR(sbuf.st_mode)) return not_tty;
+  if(!S_ISCHR(sbuf.st_mode)) return _("List member was not a TTY.");
   ret->tty = sbuf.st_rdev;
   return 0;
 }
@@ -176,13 +167,12 @@ static const char *parse_list(const char *arg, const char *(*parse_fn)(char *, s
   node = malloc(sizeof(selection_node));
   node->u = malloc(strlen(arg)*sizeof(sel_union)); /* waste is insignificant */
   node->n = 0;
-  buf = malloc(strlen(arg)+1);
-  strcpy(buf, arg);
+  buf = strdup(arg);
   /*** sanity check and count items ***/
   need_item = 1; /* true */
   items = 0;
   walk = buf;
-  err = "Improper list.";
+  err = _("Improper list.");
   do{
     switch(*walk){
     case ' ': case ',': case '\t': case '\0':
@@ -242,7 +232,7 @@ static const char *parse_sysv_option(void){
     case 'C': /* end */
       trace("-C select by process name.\n");  /* Why only HP/UX and us? */
       arg=get_opt_arg();
-      if(!arg) return "List of command names must follow -C.";
+      if(!arg) return _("List of command names must follow -C.");
       err=parse_list(arg, parse_cmd);
       if(err) return err;
       selection_list->typecode = SEL_COMM;
@@ -256,7 +246,7 @@ static const char *parse_sysv_option(void){
     case 'G': /* end */
       trace("-G select by RGID (supports names)\n");
       arg=get_opt_arg();
-      if(!arg) return "List of real groups must follow -G.";
+      if(!arg) return _("List of real groups must follow -G.");
       err=parse_list(arg, parse_gid);
       if(err) return err;
       selection_list->typecode = SEL_RGID;
@@ -269,7 +259,7 @@ static const char *parse_sysv_option(void){
     case 'J':  // specify list of job IDs in hex (IRIX) -- like HP "-R" maybe?
       trace("-J select by job ID\n");  // want a JID ("jid") for "-j" too
       arg=get_opt_arg();
-      if(!arg) return "List of jobs must follow -J.";
+      if(!arg) return _("List of jobs must follow -J.");
       err=parse_list(arg, parse_jid);
       if(err) return err;
       selection_list->typecode = SEL_JID;
@@ -297,7 +287,7 @@ static const char *parse_sysv_option(void){
     case 'O': /* end */
       trace("-O is preloaded -o.\n");
       arg=get_opt_arg();
-      if(!arg) return "Format or sort specification must follow -O.";
+      if(!arg) return _("Format or sort specification must follow -O.");
       defer_sf_option(arg, SF_U_O);
       return NULL; /* can't have any more options */
     case 'P':     /* SunOS 5 "psr" or unknown HP/UX feature */
@@ -308,7 +298,7 @@ static const char *parse_sysv_option(void){
     case 'R':    // unknown HP/UX feature, like IRIX "-J" maybe?
       trace("-R select by PRM group\n");
       arg=get_opt_arg();
-      if(!arg) return "List of PRM groups must follow -R.";
+      if(!arg) return _("List of PRM groups must follow -R.");
       err=parse_list(arg, parse_prm);
       if(err) return err;
       selection_list->typecode = SEL_PRM;
@@ -327,7 +317,7 @@ static const char *parse_sysv_option(void){
     case 'U': /* end */
       trace("-U select by RUID (supports names).\n");
       arg=get_opt_arg();
-      if(!arg) return "List of real groups must follow -U.";
+      if(!arg) return _("List of real users must follow -U.");
       err=parse_list(arg, parse_uid);
       if(err) return err;
       selection_list->typecode = SEL_RUID;
@@ -368,7 +358,7 @@ static const char *parse_sysv_option(void){
     case 'g': /* end */
       trace("-g selects by session leader OR by group name\n");
       arg=get_opt_arg();
-      if(!arg) return "List of session leaders OR effective group names must follow -g.";
+      if(!arg) return _("List of session leaders OR effective group names must follow -g.");
       err=parse_list(arg, parse_pid);
       if(!err){
         selection_list->typecode = SEL_SESS;
@@ -379,7 +369,7 @@ static const char *parse_sysv_option(void){
         selection_list->typecode = SEL_EGID;
         return NULL; /* can't have any more options */
       }
-      return "List of session leaders OR effective group IDs was invalid.";
+      return _("List of session leaders OR effective group IDs was invalid.");
     case 'j':
       trace("-j jobs format.\n");
       /* old Debian used RD_j and Digital uses JFMT */
@@ -398,7 +388,7 @@ static const char *parse_sysv_option(void){
     case 'n': /* end */
       trace("-n sets namelist file.\n");
       arg=get_opt_arg();
-      if(!arg) return "Alternate System.map file must follow -n.";
+      if(!arg) return _("Alternate System.map file must follow -n.");
       namelist_file = arg;
       return NULL; /* can't have any more options */
     case 'o': /* end */
@@ -409,13 +399,13 @@ static const char *parse_sysv_option(void){
       /* This parser looks for any excuse to ignore that braindamage.  */
       trace("-o user-defined format.\n");
       arg=get_opt_arg();
-      if(!arg) return "Format specification must follow -o.";
+      if(!arg) return _("Format specification must follow -o.");
       not_pure_unix |= defer_sf_option(arg, SF_U_o);
       return NULL; /* can't have any more options */
     case 'p': /* end */
       trace("-p select by PID.\n");
       arg=get_opt_arg();
-      if(!arg) return "List of process IDs must follow -p.";
+      if(!arg) return _("List of process IDs must follow -p.");
       err=parse_list(arg, parse_pid);
       if(err) return err;
       selection_list->typecode = SEL_PID;
@@ -424,13 +414,13 @@ static const char *parse_sysv_option(void){
     case 'r':
       trace("-r some Digital Unix thing about warnings...\n");
       trace("   or SCO's option to chroot() for new /proc and /dev.\n");
-      return "The -r option is reserved.";
+      return _("The -r option is reserved.");
       break;
 #endif
     case 's': /* end */
       trace("-s Select processes belonging to the sessions given.\n");
       arg=get_opt_arg();
-      if(!arg) return "List of session IDs must follow -s.";
+      if(!arg) return _("List of session IDs must follow -s.");
       err=parse_list(arg, parse_pid);
       if(err) return err;
       selection_list->typecode = SEL_SESS;
@@ -438,15 +428,15 @@ static const char *parse_sysv_option(void){
     case 't': /* end */
       trace("-t select by tty.\n");
       arg=get_opt_arg();
-      if(!arg) return "List of terminals (pty, tty...) must follow -t.";
+      if(!arg) return _("List of terminals (pty, tty...) must follow -t.");
       err=parse_list(arg, parse_tty);
       if(err) return err;
       selection_list->typecode = SEL_TTY;
       return NULL; /* can't have any more options */
     case 'u': /* end */
-      trace("-u select by user ID (the EUID?) (supports names).\n");
+      trace("-u select by user effective ID (supports names).\n");
       arg=get_opt_arg();
-      if(!arg) return "List of users must follow -u.";
+      if(!arg) return _("List of users must follow -u.");
       err=parse_list(arg, parse_uid);
       if(err) return err;
       selection_list->typecode = SEL_EUID;
@@ -468,7 +458,7 @@ static const char *parse_sysv_option(void){
         unix_f_option = 1;
         break;
       }
-      return "Must set personality to get -x option.";
+      return _("Must set personality to get -x option.");
     case 'y':  /* Sun's -l hack (also: Irix "lnode" resource control info) */
       trace("-y Print lnone info in UID/USER column or do Sun -l hack.\n");
       format_modifiers |= FM_y;
@@ -483,20 +473,20 @@ static const char *parse_sysv_option(void){
     case 'z':     /* select by zone */
       trace("-z secects by zone\n");
       arg=get_opt_arg();
-      if(!arg) return "List of zones (contexts, labels, whatever?) must follow -z.";
+      if(!arg) return _("List of zones (contexts, labels, whatever?) must follow -z.");
       err=parse_list(arg, parse_zone);
       if(err) return err;
       selection_list->typecode = SEL_ZONE;
       return NULL; /* can't have any more options */
 #endif
     case '-':
-      return "Embedded '-' among SysV options makes no sense.";
+      return _("Embedded '-' among SysV options makes no sense.");
       break;
     case '\0':
-      return "Please report the \"SysV \\0 can't happen\" bug.";
+      return _("Please report the \"SysV \\0 can't happen\" bug.");
       break;
     default:
-      return "Unsupported SysV option.";
+      return _("Unsupported SysV option.");
     } /* switch */
   } /* while */
   return NULL;
@@ -509,13 +499,13 @@ static const char *parse_bsd_option(void){
 
   flagptr = ps_argv[thisarg];  /* assume we _have_ a '-' */
   if(flagptr[0]=='-'){
-    if(!force_bsd) return "Can't happen!  Problem #1.";
+    if(!force_bsd) return _("Can't happen!  Problem #1.");
   }else{
     flagptr--; /* off beginning, will increment before use */
     if(personality & PER_FORCE_BSD){
-      if(!force_bsd) return "Can't happen!  Problem #2.";
+      if(!force_bsd) return _("Can't happen!  Problem #2.");
     }else{
-      if(force_bsd) return "2nd chance parse failed, not BSD or SysV.";
+      if(force_bsd) return _("2nd chance parse failed, not BSD or SysV.");
     }
   }
 
@@ -532,12 +522,12 @@ static const char *parse_bsd_option(void){
     case 'A':
       /* maybe this just does a larger malloc() ? */
       trace("A Increases the argument space (Digital Unix)\n");
-      return "Option A is reserved.";
+      return _("Option A is reserved.");
       break;
     case 'C':
       /* should divide result by 1-(e**(foo*log(bar))) */
       trace("C Use raw CPU time for %%CPU instead of decaying ave\n");
-      return "Option C is reserved.";
+      return _("Option C is reserved.");
       break;
 #endif
     case 'H':    // The FreeBSD way (NetBSD:s OpenBSD:k FreeBSD:H  -- NIH???)
@@ -557,13 +547,13 @@ static const char *parse_bsd_option(void){
     case 'N': /* end */
       trace("N Specify namelist file\n");
       arg=get_opt_arg();
-      if(!arg) return "Alternate System.map file must follow N.";
+      if(!arg) return _("Alternate System.map file must follow N.");
       namelist_file = arg;
       return NULL; /* can't have any more options */
     case 'O': /* end */
       trace("O Like o + defaults, add new columns after PID. Also sort.\n");
       arg=get_opt_arg();
-      if(!arg) return "Format or sort specification must follow O.";
+      if(!arg) return _("Format or sort specification must follow O.");
       defer_sf_option(arg, SF_B_O);
       return NULL; /* can't have any more options */
       break;
@@ -588,7 +578,7 @@ static const char *parse_bsd_option(void){
     case 'U': /* end */
       trace("U Select processes for specified users.\n");
       arg=get_opt_arg();
-      if(!arg) return "List of users must follow U.";
+      if(!arg) return _("List of users must follow U.");
       err=parse_list(arg, parse_uid);
       if(err) return err;
       selection_list->typecode = SEL_EUID;
@@ -600,7 +590,7 @@ static const char *parse_bsd_option(void){
       exit(0);
     case 'W':
       trace("W N/A get swap info from ... not /dev/drum.\n");
-      return "Obsolete W option not supported. (You have a /dev/drum?)";
+      return _("Obsolete W option not supported. (You have a /dev/drum?)");
       break;
     case 'X':
       trace("X Old Linux i386 register format\n");
@@ -636,7 +626,7 @@ static const char *parse_bsd_option(void){
       break;
     case 'h':
       trace("h Repeat header... yow.\n");
-      if(header_type) return "Only one heading option may be specified.";
+      if(header_type) return _("Only one heading option may be specified.");
       if(personality & PER_BSD_h) header_type = HEAD_MULTI;
       else                        header_type = HEAD_NONE;
       break;
@@ -651,7 +641,7 @@ static const char *parse_bsd_option(void){
       // NetBSD, and soon (?) FreeBSD: sort-by-keyword
       trace("k Specify sorting keywords.\n");
       arg=get_opt_arg();
-      if(!arg) return "Long sort specification must follow 'k'.";
+      if(!arg) return _("Long sort specification must follow 'k'.");
       defer_sf_option(arg, SF_G_sort);
       return NULL; /* can't have any more options */
     case 'l':
@@ -679,13 +669,13 @@ static const char *parse_bsd_option(void){
     case 'o': /* end */
       trace("o Specify user-defined format\n");
       arg=get_opt_arg();
-      if(!arg) return "Format specification must follow o.";
+      if(!arg) return _("Format specification must follow o.");
       defer_sf_option(arg, SF_B_o);
       return NULL; /* can't have any more options */
     case 'p': /* end */
       trace("p Select by process ID\n");
       arg=get_opt_arg();
-      if(!arg) return "List of process IDs must follow p.";
+      if(!arg) return _("List of process IDs must follow p.");
       err=parse_list(arg, parse_pid);
       if(err) return err;
       selection_list->typecode = SEL_PID;
@@ -735,13 +725,13 @@ static const char *parse_bsd_option(void){
       simple_select |= SS_B_x;
       break;
     case '-':
-      return "Embedded '-' among BSD options makes no sense.";
+      return _("Embedded '-' among BSD options makes no sense.");
       break;
     case '\0':
-      return "Please report the \"BSD \\0 can't happen\" bug.";
+      return _("Please report the \"BSD \\0 can't happen\" bug.");
       break;
     default:
-      return "Unsupported option (BSD syntax)";
+      return _("Unsupported option (BSD syntax)");
     } /* switch */
   } /* while */
   return NULL;
@@ -801,7 +791,7 @@ static const char *parse_gnu_option(void){
   {"headers",       &&case_headers},
   {"heading",       &&case_heading},
   {"headings",      &&case_headings},
-  {"help",          &&case_help},
+//{"help",          &&case_help},        /* now TRANSLATABLE ! */
   {"info",          &&case_info},
   {"lines",         &&case_lines},
   {"no-header",     &&case_no_header},
@@ -826,7 +816,7 @@ static const char *parse_gnu_option(void){
 
   s = ps_argv[thisarg]+2;
   sl = strcspn(s,":=");
-  if(sl > 15) return "Unknown gnu long option.";
+  if(sl > 15) return _("Unknown gnu long option.");
   strncpy(buf, s, sl);
   buf[sl] = '\0';
   flagptr = s+sl;
@@ -835,14 +825,18 @@ static const char *parse_gnu_option(void){
       sizeof(gnu_table_struct), compare_gnu_table_structs
   );
 
-  if(!found) return "Unknown gnu long option.";
+  if(!found) {
+    if (!strcmp(buf, the_word_help))
+      goto case_help;
+    return _("Unknown gnu long option.");
+  }
 
   goto *(found->jump);    /* See gcc extension info.  :-)   */
 
   case_Group:
     trace("--Group\n");
     arg = grab_gnu_arg();
-    if(!arg) return "List of real groups must follow --Group.";
+    if(!arg) return _("List of real groups must follow --Group.");
     err=parse_list(arg, parse_gid);
     if(err) return err;
     selection_list->typecode = SEL_RGID;
@@ -850,7 +844,7 @@ static const char *parse_gnu_option(void){
   case_User:
     trace("--User\n");
     arg = grab_gnu_arg();
-    if(!arg) return "List of real users must follow --User.";
+    if(!arg) return _("List of real users must follow --User.");
     err=parse_list(arg, parse_uid);
     if(err) return err;
     selection_list->typecode = SEL_RUID;
@@ -869,15 +863,15 @@ static const char *parse_gnu_option(void){
         return NULL;
       }
     }
-    return "Number of columns must follow --cols, --width, or --columns.";
+    return _("Number of columns must follow --cols, --width, or --columns.");
   case_cumulative:
     trace("--cumulative\n");
-    if(s[sl]) return "Option --cumulative does not take an argument.";
+    if(s[sl]) return _("Option --cumulative does not take an argument.");
     include_dead_children = 1;
     return NULL;
   case_deselect:
     trace("--deselect\n");
-    if(s[sl]) return "Option --deselect does not take an argument.";
+    if(s[sl]) return _("Option --deselect does not take an argument.");
     negate_selection = 1;
     return NULL;
   case_no_header:
@@ -889,8 +883,8 @@ static const char *parse_gnu_option(void){
   case_noheading:
   case_noheadings:
     trace("--noheaders\n");
-    if(s[sl]) return "Option --no-heading does not take an argument.";
-    if(header_type) return "Only one heading option may be specified.";
+    if(s[sl]) return _("Option --no-heading does not take an argument.");
+    if(header_type) return _("Only one heading option may be specified.");
     header_type = HEAD_NONE;
     return NULL;
   case_header:
@@ -898,35 +892,33 @@ static const char *parse_gnu_option(void){
   case_heading:
   case_headings:
     trace("--headers\n");
-    if(s[sl]) return "Option --heading does not take an argument.";
-    if(header_type) return "Only one heading option may be specified.";
+    if(s[sl]) return _("Option --heading does not take an argument.");
+    if(header_type) return _("Only one heading option may be specified.");
     header_type = HEAD_MULTI;
     return NULL;
   case_forest:
     trace("--forest\n");
-    if(s[sl]) return "Option --forest does not take an argument.";
+    if(s[sl]) return _("Option --forest does not take an argument.");
     forest_type = 'g';
     return NULL;
   case_format:
     trace("--format\n");
     arg=grab_gnu_arg();
-    if(!arg) return "Format specification must follow --format.";
+    if(!arg) return _("Format specification must follow --format.");
     defer_sf_option(arg, SF_G_format);
     return NULL;
   case_group:
     trace("--group\n");
     arg = grab_gnu_arg();
-    if(!arg) return "List of effective groups must follow --group.";
+    if(!arg) return _("List of effective groups must follow --group.");
     err=parse_list(arg, parse_gid);
     if(err) return err;
     selection_list->typecode = SEL_EGID;
     return NULL;
   case_help:
     trace("--help\n");
-    exclusive("--help");
-    fwrite(help_message,1,strlen(help_message),stdout);
-    exit(0);
-    return NULL;
+    arg = grab_gnu_arg();
+    do_help(arg, EXIT_SUCCESS);
   case_info:
     trace("--info\n");
     exclusive("--info");
@@ -936,7 +928,7 @@ static const char *parse_gnu_option(void){
   case_pid:
     trace("--pid\n");
     arg = grab_gnu_arg();
-    if(!arg) return "List of process IDs must follow --pid.";
+    if(!arg) return _("List of process IDs must follow --pid.");
     err=parse_list(arg, parse_pid);
     if(err) return err;
     selection_list->typecode = SEL_PID;
@@ -944,7 +936,7 @@ static const char *parse_gnu_option(void){
   case_ppid:
     trace("--ppid\n");
     arg = grab_gnu_arg();
-    if(!arg) return "List of process IDs must follow --ppid.";
+    if(!arg) return _("List of process IDs must follow --ppid.");
     err=parse_list(arg, parse_pid);
     if(err) return err;
     selection_list->typecode = SEL_PPID;
@@ -962,11 +954,11 @@ static const char *parse_gnu_option(void){
         return NULL;
       }
     }
-    return "Number of rows must follow --rows or --lines.";
+    return _("Number of rows must follow --rows or --lines.");
   case_sid:
     trace("--sid\n");
     arg = grab_gnu_arg();
-    if(!arg) return "Some sid thing(s) must follow --sid.";
+    if(!arg) return _("Some sid thing(s) must follow --sid.");
     err=parse_list(arg, parse_pid);
     if(err) return err;
     selection_list->typecode = SEL_SESS;
@@ -974,13 +966,13 @@ static const char *parse_gnu_option(void){
   case_sort:
     trace("--sort\n");
     arg=grab_gnu_arg();
-    if(!arg) return "Long sort specification must follow --sort.";
+    if(!arg) return _("Long sort specification must follow --sort.");
     defer_sf_option(arg, SF_G_sort);
     return NULL;
   case_tty:
     trace("--tty\n");
     arg = grab_gnu_arg();
-    if(!arg) return "List of ttys must follow --tty.";
+    if(!arg) return _("List of ttys must follow --tty.");
     err=parse_list(arg, parse_tty);
     if(err) return err;
     selection_list->typecode = SEL_TTY;
@@ -988,7 +980,7 @@ static const char *parse_gnu_option(void){
   case_user:
     trace("--user\n");
     arg = grab_gnu_arg();
-    if(!arg) return "List of effective users must follow --user.";
+    if(!arg) return _("List of effective users must follow --user.");
     err=parse_list(arg, parse_uid);
     if(err) return err;
     selection_list->typecode = SEL_EUID;
@@ -1105,7 +1097,7 @@ static const char *parse_all_options(void){
         err = parse_sysv_option();
         break;
     case ARG_BSD:
-        if(force_bsd && !(personality & PER_FORCE_BSD)) return "way bad";
+        if(force_bsd && !(personality & PER_FORCE_BSD)) return _("way bad");
       }
       prefer_bsd_defaults = 1;
       err = parse_bsd_option();
@@ -1119,11 +1111,11 @@ static const char *parse_all_options(void){
     case ARG_END:
     case ARG_FAIL:
       trace("              FAIL/END on [%s]\n",ps_argv[thisarg]);
-      return "Garbage option.";
+      return _("Garbage option.");
       break;
     default:
       printf("                  ?    %s\n",ps_argv[thisarg]);
-      return "Something broke.";
+      return _("Something broke.");
     } /* switch */
     if(err) return err;
   } /* while */
@@ -1143,16 +1135,16 @@ static const char *thread_option_check(void){
   }
 
   if(forest_type){
-    return "Thread display conflicts with forest display.";
+    return _("Thread display conflicts with forest display.");
   }
   //thread_flags |= TF_no_forest;
 
   if((thread_flags&TF_B_H) && (thread_flags&(TF_B_m|TF_U_m)))
-    return "Thread flags conflict; can't use H with m or -m.";
+    return _("Thread flags conflict; can't use H with m or -m.");
   if((thread_flags&TF_B_m) && (thread_flags&TF_U_m))
-    return "Thread flags conflict; can't use both m and -m.";
+    return _("Thread flags conflict; can't use both m and -m.");
   if((thread_flags&TF_U_L) && (thread_flags&TF_U_T))
-    return "Thread flags conflict; can't use both -L and -T.";
+    return _("Thread flags conflict; can't use both -L and -T.");
 
   if(thread_flags&TF_B_H) thread_flags |= (TF_show_proc|TF_loose_tasks);
   if(thread_flags&(TF_B_m|TF_U_m)) thread_flags |= (TF_show_proc|TF_show_task|TF_show_both);
@@ -1161,7 +1153,7 @@ static const char *thread_option_check(void){
     if(thread_flags&(TF_B_m|TF_U_m|TF_B_H)){
       // Got a thread style, so format modification is a requirement?
       // Maybe -T/-L has H thread style though. (sorting interaction?)
-      //return "Huh? Tell procps@freelists.org what you expected.";
+      //return _("Huh? Tell procps@freelists.org what you expected.");
       thread_flags |= TF_must_use;
     }else{
       // using -L/-T thread style, so format from elsewhere is OK
@@ -1231,8 +1223,19 @@ try_bsd:
   // for some non-default personalities. So "ps -ax" will parse
   // as SysV options... and you're screwed if you've been patching
   // out the friendly warning. Cut-over is likely to be in 2005.
+#ifdef BUILD_WITH_WHINE
+  // Slackware:
+  //   IMO, people can change old habits if and when user 'x' comes
+  //   along.  I still find this warning to be a POLA violation.  No
+  //   offense...  that's the beauty of open source.  You've got your
+  //   ideas about this, and I have mine, and we're allowed to
+  //   disagree.  Nothing in the UNIX or POSIX standards requires
+  //   this (annoying) warning to be displayed, and we're not
+  //   changing the actual behavior of ps in any way.  I know of no
+  //   other 'ps' that produces this message.
   if(!(personality & PER_FORCE_BSD))
-    fprintf(stderr, "Warning: bad ps syntax, perhaps a bogus '-'? See http://procps.sf.net/faq.html\n");
+    fprintf(stderr, _("Warning: bad ps syntax, perhaps a bogus '-'? See http://gitorious.org/procps/procps/blobs/master/FAQ\n"));
+#endif
   // Remember: contact procps@freelists.org
   // if you should feel tempted. Be damn sure you understand all
   // the issues. The same goes for other stuff too, BTW. Please ask.
@@ -1243,9 +1246,7 @@ try_bsd:
 
 total_failure:
   reset_parser();
-  if(personality & PER_FORCE_BSD) fprintf(stderr, "ERROR: %s\n", err2);
-  else fprintf(stderr, "ERROR: %s\n", err);
-  fwrite(help_message,1,strlen(help_message),stderr);
-  exit(1);
-  /* return 1; */ /* useless */
+  if(personality & PER_FORCE_BSD) fprintf(stderr, _("ERROR: %s\n"), err2);
+  else fprintf(stderr, _("ERROR: %s\n"), err);
+  do_help(NULL, EXIT_FAILURE);
 }
